#
# Copyright 2008    Brennan Ashton <bashton@brennanashton.com>
# 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with translate; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import turbogears as tg
from turbogears import controllers, expose, flash, validate, validators
from turbogears.widgets import WidgetsList, TextArea, CalendarDatePicker, TableForm, CheckBox, HiddenField

import time
from datetime import datetime
import sqlalchemy

def datetodays(date):
    return(int(time.mktime(date.timetuple()))/60/60/24*60*60*24)

class indivgraph_form(WidgetsList):
    start_date = CalendarDatePicker(
        label = _('Start Date'),
        name = 'start',
        calendar_lang = 'en',
        format = '%Y-%m-%d',
        validator = validators.DateTimeConverter(format="%Y-%m-%d"))
    end_date = CalendarDatePicker(
        label = _('End Date'),
        name = 'end',
        calendar_lang = 'en',
        format = '%Y-%m-%d',
        validator = validators.DateTimeConverter(format="%Y-%m-%d"))
    sNew = CheckBox(label= _('NEW'),attrs={'value':'sNEW'},name = 'status')
    sVERIFIED = CheckBox(label= _('VERIFIED'),attrs={'value':'sVERIFIED'},name = 'status')
    sMODIFIED = CheckBox(label= _('MODIFIED'),attrs={'value':'sMODIFIED'},name = 'status')
    sASSIGNED = CheckBox(label= _('ASSIGNED'),attrs={'value':'sASSIGNED'},name = 'status')
    sCLOSED = CheckBox(label= _('CLOSED'),attrs={'value':'sCLOSED'},name = 'status')
    sPOST = CheckBox(label= _('POST'),attrs={'value':'sPOST'},name = 'status')
    sON_QA = CheckBox(label= _('ON_QA'),attrs={'value':'sON_QA'},name = 'status')
    sneedinfo = CheckBox(label= _('needinfo?'),attrs={'value':'sneedinfo'},name = 'status')
    sTotal = CheckBox(label= _('Total'),attrs={'value':'sTotal'},name = 'status') 
    
class Root(controllers.RootController):
    statusdict = {'sNEW':'NEW','sVERIFIED':'VERIFIED','sMODIFIED':'MODIFIED','sASSIGNED':'ASSIGNED',
                  'sCLOSED':'CLOSED','sPOST':'POST','sON_QA':'ON_QA','sneedinfo':'needinfo?','sTotal':'Total'}
    @expose(template="triageweb.templates.welcome")
    def index(self):
        from model import triageByDate
        from turboflot import TurboFlot
        cutlist =['id','date', 'component','NEEDINFO']
        start_s=time.mktime(time.strptime('2009-02-01',"%Y-%m-%d"))
        end_s=time.mktime(time.strptime('2009-03-22',"%Y-%m-%d"))
        flotlist = []
        date_dict = {}
        date_list = triageByDate.select(sqlalchemy.and_(triageByDate.c.date >= start_s, triageByDate.c.date <= end_s)).execute()
        for record in date_list:
            date_dict[record.date] = None
        date_list = date_dict.keys()    
        headings=[elem for elem in triageByDate.c.keys() if (elem not in cutlist)]
        for heading in headings:
            datalist = []
            for day in date_list:
                triagedate = triageByDate.select(triageByDate.c.date == day).execute()
                dateh_count = 0
                for comp in triagedate:
                    dateh_count += comp[heading]
                datalist.append([day*1000,dateh_count])
            #the dates could come in any order, make sure they are ordered 
            datalist=sorted(datalist)
            flotlist.append({'label':heading,'data': datalist,
                             'lines' : { 'show' : True, 'fill' : False },
                             'points': { 'show': True, 'fill': False }
                             })
        flot = TurboFlot(flotlist,
            {
                'grid'  : { 'backgroundColor' : '#fffaff' },
                'xaxis' : { 'mode' : 'time', 'timeformat' : '%y/%m/%d' }
            }
        )
        return dict(flot=flot)

    @expose(template="triageweb.templates.triageName")
    def triagers(self,start='1990-01-01',end=time.strftime("%Y-%m-%d"),sortv='triagerName',grouponly='0'):
        from model import triageByUsers, groups
        triagegroup = [elem['name'] for elem in groups.select(groups.c.group == 'triagers').execute()]
        from collections import defaultdict
        start_s=time.mktime(time.strptime(start,"%Y-%m-%d"))
        end_s=time.mktime(time.strptime(end,"%Y-%m-%d"))
        test = {}
        cutlist =['id','date', 'triagerName','NEEDINFO']
        if(grouponly == '1'):
            triageusers = triageByUsers.select(sqlalchemy.and_(triageByUsers.c.date >= start_s, triageByUsers.c.date <= end_s, triageByUsers.c.triagerName.in_(triagegroup))).execute()
        else:
            triageusers = triageByUsers.select(sqlalchemy.and_(triageByUsers.c.date >= start_s, triageByUsers.c.date <= end_s)).execute()
        triagenset = set([elem['triagerName'] for elem in triageusers])
        for triager in triagenset:
            triageuser = triageByUsers.select(sqlalchemy.and_(triageByUsers.c.date >= start_s, triageByUsers.c.date <= end_s,triageByUsers.c.triagerName == triager)).execute()
            test[triager] = defaultdict(int)
            test[triager]['triagerName'] = triager
            for d in triageuser:
                for k in range(0,len(d)):
                    if triageByUsers.c.keys()[k] not in cutlist:
                        if d[k] == None:
                            test[triager][triageByUsers.c.keys()[k]] += 0
                        else:
                            test[triager][triageByUsers.c.keys()[k]] += d[k]
            test[triager]['Total'] = sum([test[triager][elem] for elem in test[triager] if elem != 'triagerName'])
        if sortv == 'triagerName':
            sortkey=sorted(test, key=lambda k: test[k][sortv].lower())
        else:
            sortkey=sorted(test, key=lambda k: test[k][sortv],reverse=True)

        return dict(triageusers=test,
                    headings=[elem for elem in (triageByUsers.c.keys() + ['Total']) if (elem not in cutlist) or (elem == 'triagerName')],
                    sortkey=sortkey,
                    start=start,
                    end=end,
                    sortv=sortv,
                    grouponly=grouponly)
    @expose(template="triageweb.templates.triageTime")
    def triager(self,start='1990-01-01',end=time.strftime("%Y-%m-%d"),triagenm='Fedora Update System',status=[]):
        from model import triageByUsers
        from turboflot import TurboFlot
        if str(status) == status: status = [status]
        cutlist =['id','date', 'triagerName','NEEDINFO']
        totallist = [elem for elem in triageByUsers.c.keys() if (elem not in cutlist)]
        if status != []:
            cutlist.extend([self.statusdict[elem] for elem in (set(status) ^ set(self.statusdict.keys()))])
        start_s=time.mktime(time.strptime(start,"%Y-%m-%d"))
        end_s=time.mktime(time.strptime(end,"%Y-%m-%d"))
        flotlist = []
        headings=[elem for elem in (triageByUsers.c.keys() +['Total']) if (elem not in cutlist)]
        for heading in headings:
            triageuser = triageByUsers.select(sqlalchemy.and_(triageByUsers.c.date >= start_s, triageByUsers.c.date <= end_s, triageByUsers.c.triagerName == triagenm)).execute()
            datalist = []
            for day in triageuser:
                if(heading != 'Total'):
                    datalist.append([day['date']*1000,day[heading]])
                else:
                    datalist.append([day['date']*1000,sum([day[elem] for elem in totallist if elem != 'Total'])])            #the dates could come in any order, make sure they are ordered 
            datalist=sorted(datalist)
            flotlist.append({'label':heading,'data': datalist,
                             'lines' : { 'show' : True, 'fill' : False },
                             'points': { 'show': True, 'fill': False }
                             })
        flot = TurboFlot(flotlist,
            {
                'grid'  : { 'backgroundColor' : '#fffaff' },
                'xaxis' : { 'mode' : 'time', 'timeformat' : '%y/%m/%d' }
            }
        )
        controllerform = TableForm(
                                   fields=indivgraph_form()+ [HiddenField(name='triagenm')] ,
                                   action="triager"
                                   )
        return dict(flot=flot,triagenm=triagenm,start=start,end=end,controllerform=controllerform)
    @expose(template="triageweb.templates.triageComponents")
    def components(self,start='1990-01-01',end=time.strftime("%Y-%m-%d"),sortv='component'):
        from model import triageByDate
        from collections import defaultdict
        start_s=time.mktime(time.strptime(start,"%Y-%m-%d"))
        end_s=time.mktime(time.strptime(end,"%Y-%m-%d"))
        test = {}
        cutlist =['id','date', 'component','NEEDINFO']
        components = triageByDate.select(sqlalchemy.and_(triageByDate.c.date >= start_s, triageByDate.c.date <= end_s)).execute()
        compset = set([elem['component'] for elem in components])
        for component in compset:
            triagecomp = triageByDate.select(sqlalchemy.and_(triageByDate.c.date >= start_s, triageByDate.c.date <= end_s,triageByDate.c.component == component)).execute()
            test[component] = defaultdict(int)
            test[component]['component'] = component
            for d in triagecomp:
                for k in range(0,len(d)):
                    if triageByDate.c.keys()[k] not in cutlist:
                        if d[k] == None:
                            test[component][triageByDate.c.keys()[k]] += 0
                        else:
                            test[component][triageByDate.c.keys()[k]] += d[k]
        if sortv == 'component':
            sortkey=sorted(test, key=lambda k: test[k][sortv].lower())
        else:
            sortkey=sorted(test, key=lambda k: test[k][sortv],reverse=True)
            
        return dict(triagecomp=test,
                    headings=[elem for elem in triageByDate.c.keys() if (elem not in cutlist) or (elem == 'component')],
                    sortkey=sortkey,
                    start=start,
                    end=end,
                    sortv=sortv)
    @expose(template="triageweb.templates.triageCompTime")
    def component(self,start='1990-01-01',end=time.strftime("%Y-%m-%d"),component='kernel',status=[]):
        from model import triageByDate
        from turboflot import TurboFlot
        if str(status) == status: status = [status]
        cutlist =['id','date', 'component','NEEDINFO']
        totallist = [elem for elem in triageByDate.c.keys() if (elem not in cutlist)]
        if status != []:
            cutlist.extend([self.statusdict[elem] for elem in (set(status) ^ set(self.statusdict.keys()))])
        start_s=time.mktime(time.strptime(start,"%Y-%m-%d"))
        end_s=time.mktime(time.strptime(end,"%Y-%m-%d"))
        flotlist = []
        headings=[elem for elem in (triageByDate.c.keys() + ['Total']) if (elem not in cutlist)]
        for heading in headings:
            triagecomp = triageByDate.select(sqlalchemy.and_(triageByDate.c.date >= start_s, triageByDate.c.date <= end_s, triageByDate.c.component == component)).execute()
            datalist = []
            for day in triagecomp:
                if(heading != 'Total'):
                    datalist.append([day['date']*1000,day[heading]])
                else:
                    datalist.append([day['date']*1000,sum([day[elem] for elem in totallist if elem != 'Total'])])
            #the dates could come in any order, make sure they are ordered 
            datalist=sorted(datalist)
            flotlist.append({'label':heading,'data': datalist,
                             'lines' : { 'show' : True, 'fill' : False },
                             'points': { 'show': True, 'fill': False }
                             })
        flot = TurboFlot(flotlist,
            {
                'grid'  : { 'backgroundColor' : '#fffaff' },
                'xaxis' : { 'mode' : 'time', 'timeformat' : '%y/%m/%d' }
            }
        )
        controllerform = TableForm(
                                   fields=indivgraph_form()+ [HiddenField(name='component')] ,
                                   action="component"
                                   )
        return dict(flot=flot,triagecomp=component,start=start,end=end,controllerform=controllerform)
    @expose(template="triageweb.templates.triageComponentLevels")
    def componentlevels(self,start='1990-01-01',end=time.strftime("%Y-%m-%d"),sortv='component',status=[],extra='test'):
        from model import triageByDateTotal
        start_s=time.mktime(time.strptime(start,"%Y-%m-%d"))
        end_s=time.mktime(time.strptime(end,"%Y-%m-%d"))
        controllerform = TableForm(
                                   fields=indivgraph_form()+ [HiddenField(name='component')] ,
                                   action="componentlevels"
                                   )
        return dict(controllerform=controllerform,start=start,end=end,extra=extra)
        
